/* "Null" renderer implementation.
 * Useful for headless testing */

#include "gfx/renderer_api.h"
#include "runtime/runtime.h"

#define RETURN_HANDLE_STUB2(type, initial)                                                         \
    static unsigned int s_next = (initial);                                                        \
    return (type) { .index = (s_next++) % RT_RENDERER_BACKEND_HANDLE_MAX_INDEX, .version = 1 }

#define RETURN_HANDLE_STUB(type) RETURN_HANDLE_STUB2(type, 1)

#define RETURN_HANDLE_ARRAY_STUB2(out, count, initial)                                             \
    static unsigned int s_next = (initial);                                                        \
    for (uint32_t i = 0; i < (count); ++i) {                                                       \
        (out)[i].index   = (s_next++) % RT_RENDERER_BACKEND_HANDLE_MAX_INDEX;                      \
        (out)[i].version = 1;                                                                      \
    }

#define RETURN_HANDLE_ARRAY_STUB(out, count) RETURN_HANDLE_ARRAY_STUB2(out, count, 1)

void RT_RENDERER_API_FN(RegisterCVars)(void) {
}

rt_result RT_RENDERER_API_FN(Init)(const rt_renderer_init_info *info) {
    RT_UNUSED(info);
    return RT_SUCCESS;
}

void RT_RENDERER_API_FN(Shutdown)(void) {
}

void RT_RENDERER_API_FN(BeginFrame)(unsigned int frame_id) {
    RT_UNUSED(frame_id);
}

void RT_RENDERER_API_FN(EndFrame)(unsigned int frame_id) {
    RT_UNUSED(frame_id);
}

rt_pipeline_handle RT_RENDERER_API_FN(CompilePipeline)(const rt_pipeline_info *info) {
    RT_UNUSED(info);
    RETURN_HANDLE_STUB(rt_pipeline_handle);
}

void RT_RENDERER_API_FN(DestroyPipeline)(rt_pipeline_handle handle) {
    RT_UNUSED(handle);
}

rt_render_target_handle RT_RENDERER_API_FN(CreateRenderTarget)(const rt_render_target_info *info) {
    RT_UNUSED(info);
    RETURN_HANDLE_STUB2(rt_render_target_handle, 2);
}

rt_render_target_handle RT_RENDERER_API_FN(GetSwapchainRenderTarget)(void) {
    return (rt_render_target_handle){.index = 1, .version = 1};
}

void RT_RENDERER_API_FN(DestroyRenderTarget)(rt_render_target_handle handle) {
    RT_UNUSED(handle);
}

rt_result RT_RENDERER_API_FN(AllocCommandBuffers)(uint32_t count,
                                                  const rt_alloc_command_buffer_info *info,
                                                  rt_command_buffer_handle *p_command_buffers) {
    RT_UNUSED(info);
    RETURN_HANDLE_ARRAY_STUB(p_command_buffers, count)
    return RT_SUCCESS;
}

rt_result RT_RENDERER_API_FN(SubmitCommandBuffers)(rt_gpu_queue queue,
                                                   const rt_submit_command_buffers_info *info) {
    RT_UNUSED(queue);
    RT_UNUSED(info);
    return RT_SUCCESS;
}

rt_result RT_RENDERER_API_FN(CreateSemaphores)(uint32_t count,
                                               const rt_gpu_semaphore_info *info,
                                               rt_gpu_semaphore_handle *p_semaphores) {
    RT_UNUSED(info);
    RETURN_HANDLE_ARRAY_STUB2(p_semaphores, count, 3)
    return RT_SUCCESS;
}

void RT_RENDERER_API_FN(DestroySemaphores)(uint32_t count, rt_gpu_semaphore_handle *semaphores) {
    RT_UNUSED(count);
    RT_UNUSED(semaphores);
}

/* NOTE(Kevin): It might become necessary to actually track the value, to correctly simulate gpu
 * behaviour */
uint64_t RT_RENDERER_API_FN(GetSemaphoreValue)(rt_gpu_semaphore_handle sem) {
    RT_UNUSED(sem);
    return 0;
}

rt_gpu_semaphore_handle RT_RENDERER_API_FN(GetSwapchainAvailableSemaphore)(void) {
    return (rt_gpu_semaphore_handle){.index = 1, .version = 1};
}

rt_gpu_semaphore_handle RT_RENDERER_API_FN(GetRenderFinishedSemaphore)(void) {
    return (rt_gpu_semaphore_handle){.index = 2, .version = 1};
}

void RT_RENDERER_API_FN(CmdBeginPass)(rt_command_buffer_handle cmd,
                                      const rt_cmd_begin_pass_info *info) {
    RT_UNUSED(cmd);
    RT_UNUSED(info);
}

void RT_RENDERER_API_FN(CmdEndPass)(rt_command_buffer_handle cmd) {
    RT_UNUSED(cmd);
}

void RT_RENDERER_API_FN(CmdTransitionRenderTarget)(rt_command_buffer_handle cmd,
                                                   rt_render_target_handle target,
                                                   rt_render_target_state state) {
    RT_UNUSED(cmd);
    RT_UNUSED(target);
    RT_UNUSED(state);
}
