#ifndef RT_VK_GPU_H
#define RT_VK_GPU_H

#include <volk/volk.h>

#define VMA_STATIC_VULKAN_FUNCTIONS 0
#define VMA_DYNAMI_VULKAN_FUNCTIONS 0
#include <vma/vk_mem_alloc.h>

#include "gfx/renderer_api.h"

/* Minimum supported value of g_gpu.max_frames_in_flight */
#define RT_VK_MIN_SUPPORTED_FRAMES_IN_FLIGHT 2

/* Maximum supported number of frames in flight.
 * The actually configured value is contained in g_gpu. */
#define RT_VK_MAX_SUPPORTED_FRAMES_IN_FLIGHT 3

#ifdef _WIN32
struct HINSTANCE__;
struct HWND__;
#elif defined(RT_USE_XLIB)
struct _XDisplay;
#endif

typedef struct {
#ifdef _WIN32
    struct HINSTANCE__ *hInstance;
    struct HWND__ *hWnd;
#elif defined(RT_USE_XLIB)
    struct _XDisplay *display;
    unsigned long window;
#endif
} rt_native_window;

typedef struct {
    uint32_t swapchain_image_index;
    VkSemaphore image_available;
    VkSemaphore render_finished;
    VkSemaphore swapchain_transitioned;
} rt_frame_data;

typedef struct {
    VkInstance instance;
    VkDebugUtilsMessengerEXT messenger;
    VkAllocationCallbacks *alloc_cb;
    VkPhysicalDevice phys_device;
    VkDevice device;
    VkSurfaceKHR surface;
    VkQueue graphics_queue;
    VkQueue compute_queue;
    VkQueue present_queue;
    VkQueue transfer_queue;
    uint32_t graphics_family;
    uint32_t compute_family;
    uint32_t present_family;
    uint32_t transfer_family;

    rt_native_window native_window;

    VkPhysicalDeviceDescriptorIndexingProperties descriptor_indexing_props;
    VkPhysicalDeviceProperties phys_device_props;
    VkPhysicalDeviceDescriptorIndexingFeatures descriptor_indexing_features;
    VkPhysicalDeviceFeatures phys_device_features;

    VmaAllocator allocator;

    unsigned int max_frames_in_flight;
    unsigned int current_frame_id;

    rt_frame_data frames[RT_VK_MAX_SUPPORTED_FRAMES_IN_FLIGHT];
} rt_vk_gpu;

#ifndef RT_VK_DONT_DEFINE_GPU_GLOBAL
extern rt_vk_gpu g_gpu;


RT_INLINE rt_frame_data *rtGetFrameData(unsigned int frame_id) {
    return &g_gpu.frames[frame_id % g_gpu.max_frames_in_flight];
}

#endif

/* Helper functions */

#define RT_VK_CHECK(expr)                                                                          \
    do {                                                                                           \
        VkResult res = expr;                                                                       \
        if (res != VK_SUCCESS) {                                                                   \
            rtReportError("vk", "Vulkan command failed with error %u.\nCommand: %s", res, #expr);  \
        }                                                                                          \
    } while (0)

VkFormat rtPixelFormatToVkFormat(rt_pixel_format format);

VkSampleCountFlagBits rtSampleCountToFlags(unsigned int count);

VkQueue rtGetQueue(rt_gpu_queue queue);

const char *rtVkFormatToString(VkFormat format);

#endif
