#ifndef RT_VK_RENDER_TARGETS_H
#define RT_VK_RENDER_TARGETS_H

#include "gpu.h"
#include "gfx/renderer_api.h"

/* Must match RT_VK_MAX_SWAPCHAIN_IMAGES */
#define RT_VK_RENDER_TARGET_MAX_IMAGES 3

typedef enum {
    RT_RENDER_TARGET_MATCH_SWAPCHAIN_SIZE = 0x01,
    RT_RENDER_TARGET_MATCH_SWAPCHAIN_FORMAT = 0x02,
} rt_render_target_match_swapchain_flags;

typedef struct {
    VkImage image[RT_VK_RENDER_TARGET_MAX_IMAGES];
    VkImageView view[RT_VK_RENDER_TARGET_MAX_IMAGES];
    VmaAllocation allocation[RT_VK_RENDER_TARGET_MAX_IMAGES];
    rt_render_target_state states[RT_VK_RENDER_TARGET_MAX_IMAGES];
    VkSampleCountFlagBits sample_count;
    VkFormat format;
    VkExtent2D extent;
    VkImageUsageFlagBits usage;
    VkImageAspectFlags aspect;
    unsigned int image_count;
    rt_render_target_match_swapchain_flags match_swapchain;
} rt_render_target;

rt_render_target *rtGetRenderTarget(rt_render_target_handle handle);

/* Update the render target that represents the swap chain */
void rtUpdateSwapchainRenderTarget(void);

/* Update render targets that match the swap chain*/
void rtUpdateRenderTargetsFromSwapchain(uint32_t image_count, VkFormat format, VkExtent2D extent);

#endif
