#ifndef RT_DS_H
#define RT_DS_H

/* Datastructure Library */

#include "runtime.h"

#ifdef __cplusplus
extern "C" {
#endif

/* A minheap.
 *
 * The memory pointed to by keys and values is not owned by the minheap
 * and instead provided by the caller. */
typedef struct {
    int *keys;
    void *values;
    size_t value_size;
    size_t capacity;
    int size;
} rt_minheap;

/* Comparison function for rtMinheapUpdate.
 * The function should return 0 if *a and *b are considered equal,
 * and a different (non-zero) value if they are non-equal.
 *
 * Note that memcmp fits this requirement.
 */
typedef int rt_minheap_cmp_fn(const void *a, const void *b, size_t n);

/* Takes the arrays and re-orders the values to create a minheap. */
RT_DLLEXPORT rt_minheap
rtCreateMinheap(int *keys, void *values, size_t value_size, size_t capacity, int initial_size);

/* Copies the value with the smallest key to min_value */
RT_DLLEXPORT void rtMinheapPeek(const rt_minheap *minheap, void *min_value);

/* Copies the value with the smallest key to min_value and removes it from the heap */
RT_DLLEXPORT void rtMinheapPop(rt_minheap *minheap, void *min_value);

/* Pushes a new value into the minheap */
RT_DLLEXPORT void rtMinheapPush(rt_minheap *minheap, int key, const void *value);

/* Changes the key of an existing value, or inserts it, if it's not found.
 *
 * Uses memcmp if cmp is NULL. */
RT_DLLEXPORT void
rtMinheapUpdate(rt_minheap *minheap, const void *value, int new_key, rt_minheap_cmp_fn *cmp);

static RT_INLINE int rtMinheapIsEmpty(rt_minheap *minheap) {
    return minheap->size == 0;
}

#ifdef __cplusplus
}
#endif

#endif
